@@ -4,7 +4,7 @@ Name: filter_stab.py
 Porpose: Show dialog to get vidstab data based on FFmpeg syntax
 Compatibility: Python3, wxPython Phoenix
 Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
 license: GPL3
 Rev: Apr.23.2024
 Code checker: flake8, pylint
@@ -540,7 +540,7 @@ class VidstabSet(wx.Dialog):
                                    )
         caption = _("Videomass - Loading...")
         msg = _("Please wait,\nThis process will take a few seconds.")
-        dlgload = PopupDialog(self, caption, msg, thread)
+        dlgload = PopupDialog(self, caption, msg)
         dlgload.ShowModal()
         thread.join()  # wait end thread
         error = thread.status
@@ -695,8 +695,10 @@ class VidstabSet(wx.Dialog):
         see <https://docs.python.org/3.8/library/webbrowser.html>
 
         """
-        page = ('https://jeanslack.github.io/Videomass/User-guide'
-                '/Video_filters_en.pdf')
+        page = ('https://jeanslack.github.io/Videomass/User-guide/'
+                'Video_filters_en.pdf#%5B%7B%22num%22%3A28%2C%22gen'
+                '%22%3A0%7D%2C%7B%22name%22%3A%22XYZ%22%7D%2C56.7%2C785.'
+                '189%2C0%5D')
 
         webbrowser.open(page)
     # ------------------------------------------------------------------#
