@@ -4,9 +4,9 @@ FileName: acodecs.py
 Porpose: Contains audio functionality for A/V Conversions
 Compatibility: Python3, wxPython4 Phoenix
 Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
 license: GPL3
-Rev: Apr.08.2024
+Rev: July.26.2025
 Code checker: flake8, pylint
 
 This file is part of Videomass.
@@ -33,7 +33,7 @@ from videomass.vdms_dialogs.audioproperties import AudioProperties
 from videomass.vdms_utils.utils import get_volume_data
 from videomass.vdms_io.io_tools import volume_detect_process
 from videomass.vdms_dialogs.shownormlist import AudioVolNormal
-from videomass.vdms_io.io_tools import stream_play
+from videomass.vdms_threads.ffplay_file import FilePlay
 
 
 class AudioEncoders(scrolled.ScrolledPanel):
@@ -66,8 +66,6 @@ class AudioEncoders(scrolled.ScrolledPanel):
                             'ogg', 'mp3', 'opus', 'copy', 'mute'),
                   ('webm'): ('default', None, None, None, None, None, 'ogg',
                              None, 'opus', 'copy', 'mute'),
-                  ('ogv'): ('default', None, 'flac', None, None, None, 'ogg',
-                            None, 'opus', 'copy', 'mute'),
                   ('wav'): (None, 'wav', None, None, None, None, None, None,
                             None, 'copy', None),
                   ('mp3'): (None, None, None, None, None, None, None, 'mp3',
@@ -149,7 +147,9 @@ class AudioEncoders(scrolled.ScrolledPanel):
         sizproper.Add(txtAinmap, 0, wx.LEFT | wx.CENTRE, 20)
         self.cmb_A_inMap = wx.ComboBox(self, wx.ID_ANY,
                                        choices=['Auto', '1', '2', '3',
-                                                '4', '5', '6', '7', '8'],
+                                                '4', '5', '6', '7', '8', '9',
+                                                '10', '11', '12', '13', '14',
+                                                '15', '16',],
                                        size=(-1, -1), style=wx.CB_DROPDOWN
                                        | wx.CB_READONLY,
                                        )
@@ -197,7 +197,8 @@ class AudioEncoders(scrolled.ScrolledPanel):
         grid_peak.Add(self.btn_stat, 0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         self.lab_amplitude = wx.StaticText(self, wx.ID_ANY, _("Target level:"))
-        grid_peak.Add(self.lab_amplitude, 0, wx.LEFT | wx.CENTRE, 20)
+        grid_peak.Add(self.lab_amplitude, 0, wx.LEFT
+                      | wx.ALIGN_CENTER_VERTICAL, 20)
         self.spin_target = FS.FloatSpin(self, wx.ID_ANY,
                                         min_val=-99.0, max_val=0.0,
                                         increment=0.5, value=-1.0,
@@ -214,7 +215,7 @@ class AudioEncoders(scrolled.ScrolledPanel):
         grid_ebu = wx.FlexGridSizer(1, 6, 0, 0)
         sizer_a_normaliz.Add(grid_ebu, 0, wx.ALL | wx.CENTRE, 5)
         lab_i = wx.StaticText(self, wx.ID_ANY, ("IL:"))
-        grid_ebu.Add(lab_i, 0, wx.CENTRE, 0)
+        grid_ebu.Add(lab_i, 0, wx.ALIGN_CENTER_VERTICAL, 0)
         self.spin_i = FS.FloatSpin(self, wx.ID_ANY,
                                    min_val=-70.0, max_val=-5.0,
                                    increment=0.5, value=-16.0,
@@ -224,7 +225,7 @@ class AudioEncoders(scrolled.ScrolledPanel):
         grid_ebu.Add(self.spin_i, 0, wx.LEFT | wx.CENTRE, 5)
 
         lab_tp = wx.StaticText(self, wx.ID_ANY, ("TP:"))
-        grid_ebu.Add(lab_tp, 0, wx.LEFT | wx.CENTRE, 20)
+        grid_ebu.Add(lab_tp, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 20)
         self.spin_tp = FS.FloatSpin(self, wx.ID_ANY,
                                     min_val=-9.0, max_val=0.0,
                                     increment=0.5, value=-1.5,
@@ -234,7 +235,7 @@ class AudioEncoders(scrolled.ScrolledPanel):
         grid_ebu.Add(self.spin_tp, 0, wx.LEFT | wx.CENTRE, 5)
 
         lab_lra = wx.StaticText(self, wx.ID_ANY, ("LRA:"))
-        grid_ebu.Add(lab_lra, 0, wx.LEFT | wx.CENTRE, 20)
+        grid_ebu.Add(lab_lra, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 20)
         self.spin_lra = FS.FloatSpin(self, wx.ID_ANY,
                                      min_val=1.0, max_val=20.0,
                                      increment=0.5, value=11.0,
@@ -267,22 +268,22 @@ class AudioEncoders(scrolled.ScrolledPanel):
                  '(when switch to RMS) in dBFS. From -99.0 to +0.0; default '
                  'for PEAK level is -1.0; default for RMS is -20.0'))
         self.spin_target.SetToolTip(tip)
-        tip = (_('Normally on a video with correctly indexed streams and '
-                 'having one or more audio streams, the first audio stream '
-                 'should be indexed at "1", the second at "2" and so on (the '
-                 'video stream on the other hand is always indexed at "0"). '
-                 'In this case it is recommended to select the desired stream '
-                 'by setting a specific index, e.g "1" for for the first '
-                 'available audio stream.\n\nSet this control to "Auto" if '
-                 'the source file is simply an audio track.'))
+        tip = (_('"Auto", lets FFmpeg select the audio stream to process '
+                 '(usually the first audio stream). If the source file(s) '
+                 'is just an audio track, it\'s recommend to always set this '
+                 'control to "Auto."\n\n"1-16", if a video file contains '
+                 'more than one audio stream, you can select a specific '
+                 'one, e.g "1" for the first available audio stream, '
+                 '"2" for the second audio stream, and so on.'))
         self.cmb_A_inMap.SetToolTip(tip)
         tip = (_('"Auto" keeps all audio streams and processes only the one '
                  'selected by the "Index Selection" control.\n\n'
-                 '"All" processes all audio streams in a video with the '
-                 'properties of the one selected by the "Index Selection" '
-                 'control.\n\n"Index Only" processes only the audio stream '
-                 'selected by the "Index Selection" control and removes all '
-                 'others from the video'))
+                 '"All" processes all audio streams with the properties '
+                 'of the one selected by the "Index Selection" control. '
+                 'Don\'t use "All" to export audio files from videos.\n\n'
+                 '"Index Only" processes only the audio stream selected by '
+                 'the "Index Selection" control and removes all others '
+                 'from the output video.'))
         self.cmb_A_outMap.SetToolTip(tip)
         tip = (_('Integrated Loudness Target in LUFS. '
                  'From -70.0 to -5.0, default is -16.0'))
@@ -462,19 +463,36 @@ class AudioEncoders(scrolled.ScrolledPanel):
             afilter = (f'-af loudnorm=I={str(self.spin_i.GetValue())}'
                        f':LRA={str(self.spin_lra.GetValue())}'
                        f':TP={str(self.spin_tp.GetValue())}')
+        else:
+            return None
+
+        self.build_playfile_command(afilter,
+                                    idx,
+                                    self.maindata.file_src[fileget[1]]
+                                    )
+        return None
+    # ------------------------------------------------------------------#
+
+    def build_playfile_command(self, afilter, index, filepath):
+        """
+        Build ffplay command for playback
+        """
+        autoexit = '-autoexit' if self.maindata.autoexit else ''
 
         if self.maindata.checktimestamp:
-            args = (f'-showmode waves -vf "{self.maindata.cmdtimestamp}" '
-                    f'{afilter} {idx}')
+            args = (f'-showmode waves {autoexit} -i "{filepath}" '
+                    f'{self.maindata.time_seq} '
+                    f'-vf "{self.maindata.cmdtimestamp}" {afilter} {index}')
         else:
-            args = f'{afilter} {idx}'
-
-        stream_play(self.maindata.file_src[fileget[1]],
-                    self.maindata.time_seq,
-                    args,
-                    self.maindata.autoexit
-                    )
-        return None
+            args = (f'-showmode waves {autoexit} -i "{filepath}" '
+                    f'{self.maindata.time_seq} '
+                    f'{afilter} {index}')
+        try:
+            with open(filepath, encoding='utf-8'):
+                FilePlay(args)
+        except IOError:
+            wx.MessageBox(_("Invalid or unsupported file:  %s") % (filepath),
+                          "Videomass", wx.ICON_EXCLAMATION, self)
     # ------------------------------------------------------------------#
 
     def get_audio_stream(self, fileselected):
@@ -489,14 +507,11 @@ class AudioEncoders(scrolled.ScrolledPanel):
         """
         selected = self.maindata.data_files[fileselected[1]].get('streams')
         isaudio = [a for a in selected if 'audio' in a.get('codec_type')]
+        idx = self.cmb_A_inMap.GetValue()
 
         if isaudio:
-            if not self.cmb_A_inMap.GetValue() == 'Auto':  # 1 to 8
-                if [v for v in selected if 'video' in v.get('codec_type')]:
-                    idx = int(self.cmb_A_inMap.GetValue())
-                else:
-                    idx = int(self.cmb_A_inMap.GetValue()) - 1
-                if not [x for x in isaudio if x.get('index') == idx]:
+            if idx.isdigit():  # not Auto
+                if not len(isaudio) - 1 == int(idx) - 1:
                     wx.MessageBox(_('Selected index does not exist or '
                                     'does not contain any audio streams'),
                                   'Videomass', wx.ICON_INFORMATION, self)
@@ -579,13 +594,15 @@ class AudioEncoders(scrolled.ScrolledPanel):
                     self.opt["AudioCodec"] = ["", ""]
 
                 elif audiocodec == "Copy":
-                    self.normalize_default()
+                    self.rdbx_normalize.SetSelection(0)
+                    self.on_normalize(self)
                     _param(False, False)
                     amap = f'-c:a:{self.opt["AudioMap"][1]}'
                     self.opt["AudioCodec"] = [amap, v]
 
-                elif audiocodec == _("No Audio"):
-                    self.normalize_default()
+                elif audiocodec == "No Audio":
+                    self.rdbx_normalize.SetSelection(0)
+                    self.on_normalize(self)
                     self.opt["AudioCodec"] = ["", v]
                     _param(False, False)
                     # break
@@ -780,8 +797,8 @@ class AudioEncoders(scrolled.ScrolledPanel):
         <https://superuser.com/questions/323119/how-can-i-normalize-audio-
         using-ffmpeg?utm_medium=organic>
         """
-        if not self.maindata.filedropselected:
-            wx.MessageBox(_("Have to select an item in the file list first"),
+        if not self.maindata.data_files:
+            wx.MessageBox(_("You need to import at least one file"),
                           'Videomass', wx.ICON_INFORMATION, self)
             return
 
@@ -802,6 +819,8 @@ class AudioEncoders(scrolled.ScrolledPanel):
             target = "PEAK"
         elif self.rdbx_normalize.GetSelection() == 2:  # RMS
             target = "RMS"
+        else:
+            return
 
         del self.opt["PEAK"][:]
         del self.opt["RMS"][:]
@@ -832,6 +851,8 @@ class AudioEncoders(scrolled.ScrolledPanel):
             title = _('PEAK-based volume statistics')
         elif self.rdbx_normalize.GetSelection() == 2:  # RMS
             title = _('RMS-based volume statistics')
+        else:
+            return
 
         if self.btn_voldect.IsEnabled():
             self.on_analyzes(self)
