@@ -4,9 +4,9 @@ Name: configurator.py
 Porpose: Set Videomass configuration on startup
 Compatibility: Python3
 Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
-Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
+Copyleft - 2025 Gianluca Pernigotto <jeanlucperni@gmail.com>
 license: GPL3
-Rev: Apr.23.2024
+Rev: Aug.14.2025
 Code checker: flake8, pylint
 
  This file is part of Videomass.
@@ -26,8 +26,6 @@ Code checker: flake8, pylint
 """
 import os
 import sys
-import site
-import shutil
 import platform
 from videomass.vdms_utils.utils import copydir_recursively
 from videomass.vdms_sys.settings_manager import ConfigManager
@@ -37,7 +35,7 @@ def msg(arg):
     """
     print logging messages during startup
     """
-    print('Info:', arg)
+    print(arg)
 
 
 def create_dirs(dirname, fconf):
@@ -51,41 +49,50 @@ def create_dirs(dirname, fconf):
     if not os.path.exists(dirname):
         try:
             os.makedirs(dirname, mode=0o777)
-        except FileExistsError as err:
+        except Exception as err:
             return {'ERROR': err}
-        except OSError as err:
-            os.remove(fconf)  # force to restart on deleting
-            thismsg = ('Please try restarting Videomass to '
-                       'restore default settings now.')
-            return {'ERROR': f'{err}\n{thismsg}'}
 
     return {'R': None}
 
 
-def restore_presets_dir(dirconf, srcpath):
+def restore_dirconf(dirconf, srcdata, portable):
     """
-    Restore preset directory from source if it doesn't exist.
+    This function is responsible for restoring the
+    configuration directory if it is missing and
+    populating it with its essential files.
     Returns dict:
         key == 'R'
         key == ERROR (if any errors)
     """
+    if not os.path.exists(dirconf):  # create the configuration directory
+        try:
+            os.mkdir(dirconf, mode=0o777)
+        except FileNotFoundError as err:  # parent directory does not exist
+            return {'ERROR': err}
+
     if not os.path.exists(os.path.join(dirconf, "presets")):
-        # try to restoring presets directory on videomass dir
-        drest = copydir_recursively(os.path.join(srcpath, "presets"), dirconf)
+        drest = copydir_recursively(os.path.join(srcdata, "presets"), dirconf)
         if drest:
             return {'ERROR': drest}
 
+    if portable:
+        transoutdir = os.path.join(dirconf, "Media", "Transcoding")
+        try:
+            if not os.path.exists(transoutdir):
+                os.makedirs(transoutdir, mode=0o777)
+        except Exception as err:
+            return {'ERROR': err}
+
     return {'R': None}
 
 
-def get_options(dirconf, fileconf, srcpath, makeportable):
+def get_options(fileconf, makeportable):
     """
     Check the application options. Reads the `settings.json`
     file; if it does not exist or is unreadable try to restore
-    it. If `dirconf` does not exist try to restore both `dirconf`
-    and `settings.json`. If VERSION is not the same as the version
-    readed, it adds new missing items while preserving the old ones
-    with the same values.
+    it. If VERSION is not the same as the version readed, it adds
+    new missing items while preserving the old ones with the same
+    values.
 
     Returns dict:
         key == 'R'
@@ -94,59 +101,28 @@ def get_options(dirconf, fileconf, srcpath, makeportable):
     conf = ConfigManager(fileconf, makeportable)
     version = ConfigManager.VERSION
 
-    if os.path.exists(dirconf):  # i.e ~/.conf/videomass dir
-        if os.path.isfile(fileconf):
-            data = {'R': conf.read_options()}
-            if not data['R']:
-                conf.write_options()
-                data = {'R': conf.read_options()}
-            if float(data['R']['confversion']) != version:  # conf version
-                data['R']['confversion'] = version
-                new = ConfigManager.DEFAULT_OPTIONS  # model
-                data = {'R': {**new, **data['R']}}
-                conf.write_options(**data['R'])
-        else:
+    if os.path.isfile(fileconf):
+        data = {'R': conf.read_options()}
+        if not data['R']:
             conf.write_options()
             data = {'R': conf.read_options()}
+        if float(data['R']['confversion']) != version:  # conf version
+            data['R']['confversion'] = version
+            new = ConfigManager.DEFAULT_OPTIONS  # model
+            data = {'R': {**new, **data['R']}}
+            conf.write_options(**data['R'])
+    else:
+        conf.write_options()
+        data = {'R': conf.read_options()}
 
-    else:  # try to restore entire configuration directory
-        dconf = copydir_recursively(srcpath,
-                                    os.path.dirname(dirconf),
-                                    "videomass",
-                                    )
-        if dconf:
-            data = {'ERROR': dconf}
-        else:
-            conf.write_options()
-            data = {'R': conf.read_options()}
+    diff = conf.default_outputdirs(**data['R'])
+    if diff != data['R']:
+        conf.write_options(**diff)  # write default outputdirs
+        data = {'R': conf.read_options()}
 
     return data
 
 
-def get_pyinstaller():
-    """
-    Get pyinstaller-based package attributes to determine
-    how to use sys.executable
-    When a bundled app starts up, the bootloader sets the sys.frozen
-    attribute and stores the absolute path to the bundle folder
-    in sys._MEIPASS.
-    For a one-folder bundle, this is the path to that folder.
-    For a one-file bundle, this is the path to the temporary folder
-    created by the bootloader.
-    """
-    if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
-        frozen, meipass = True, True
-        this = getattr(sys, '_MEIPASS', os.path.abspath(__file__))
-        data_locat = this
-
-    else:
-        frozen, meipass = False, False
-        this = os.path.realpath(os.path.abspath(__file__))
-        data_locat = os.path.dirname(os.path.dirname(this))
-
-    return frozen, meipass, this, data_locat
-
-
 def conventional_paths():
     """
     Establish the conventional paths based on OS
@@ -192,9 +168,6 @@ def portable_paths(portdirname):
     cache_dir = os.path.join(dir_conf, 'cache')  # updates executable
     trash_dir = os.path.join(dir_conf, "Trash")
 
-    if not os.path.exists(dir_conf):
-        os.makedirs(dir_conf, mode=0o777)
-
     return file_conf, dir_conf, log_dir, cache_dir, trash_dir
 
 
@@ -270,17 +243,14 @@ def get_color_scheme(theme):
     return c_scheme
 
 
-def data_location(args):
+def data_location(kwargs):
     """
     Determines data location and modes to make the app
     portable, fully portable or using conventional paths.
-    return data location.
+    Returns data location dict.
     """
-    frozen, meipass, this, locat = get_pyinstaller()
-    workdir = os.path.dirname(os.path.dirname(os.path.dirname(this)))
-
-    if args['make_portable']:
-        portdir = args['make_portable']
+    if kwargs['make_portable']:
+        portdir = kwargs['make_portable']
         (conffile, confdir, logdir,
          cachedir, trash_dir) = portable_paths(portdir)
     else:
@@ -291,132 +261,72 @@ def data_location(args):
             "logdir": logdir,
             "cachedir": cachedir,
             "trashdir_default": trash_dir,
-            "this": this,
-            "frozen": frozen,
-            "meipass": meipass,
-            "locat": locat,
-            "localepath": os.path.join(locat, 'locale'),
-            "workdir": workdir,
-            "srcpath": os.path.join(locat, 'share'),
-            "icodir": os.path.join(locat, 'art', 'icons'),
-            "ffmpeg_pkg": os.path.join(locat, 'FFMPEG'),
             }
 
 
 class DataSource():
     """
-    DataSource class determines the Videomass's configuration
-    according to the used Operating System (Linux/*BSD, MacOS, Windows).
-    Remember that all specifications defined in this class refer
-    only to the following packaging methods:
-
-        - Videomass Python package by PyPi (cross-platform, multiuser/user)
-        - Linux's distributions packaging (multiuser)
-        - Bundled app by pyinstaller (windowed for MacOs and Windows)
-        - AppImage for Linux (user)
-
-        * multiuser: system installation
-        * user: local installation
+    DataSource class determines the Videomass's
+    configuration according to the used Operating
+    System and installed package.
 
     """
-    # -------------------------------------------------------------------
-
     def __init__(self, kwargs):
         """
-        Having the pathnames returned by `dataloc` it
-        performs the initialization described in DataSource.
+        Having the pathnames returned by `dataloc`
+        it performs the initialization described in
+        DataSource.
 
         """
         self.dataloc = data_location(kwargs)
         self.relativepaths = bool(kwargs['make_portable'])
         self.makeportable = kwargs['make_portable']
-        self.apptype = None  # appimage, pyinstaller on None
-        launcher = os.path.isfile(f"{self.dataloc['workdir']}/launcher")
-
-        if self.dataloc['frozen'] and self.dataloc['meipass'] or launcher:
-            msg(f"frozen={self.dataloc['frozen']} "
-                f"meipass={self.dataloc['meipass']} "
-                f"launcher={launcher}")
-
-            self.apptype = 'pyinstaller' if not launcher else None
-            self.prg_icon = f"{self.dataloc['icodir']}/videomass.png"
-
-        elif ('/tmp/.mount_' in sys.executable or os.path.exists(
-              os.path.dirname(os.path.dirname(os.path.dirname(
-                  sys.argv[0])))
-              + '/AppRun')):
-            # embedded on python appimage
-            msg('Embedded on python appimage')
-            self.apptype = 'appimage'
-            userbase = os.path.dirname(os.path.dirname(sys.argv[0]))
-            pixmaps = '/share/pixmaps/videomass.png'
-            self.prg_icon = os.path.join(userbase + pixmaps)
 
+        if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
+            sitepkg = getattr(sys, '_MEIPASS', os.path.abspath(__file__))
+            srcdata = sitepkg
+            self.dataloc["app"] = 'pyinstaller'
+            msg('Info: Stand-Alone application bundle (build by pyinstaller)')
         else:
-            binarypath = shutil.which('videomass')
-            if platform.system() == 'Windows':  # any other packages
-                exe = binarypath if binarypath else sys.executable
-                msg(f'Win32 executable={exe}')
-                # HACK check this
-                # dirname = os.path.dirname(sys.executable)
-                # pythonpath = os.path.join(dirname, 'Script', 'videomass')
-                # self.icodir = dirname + '\\share\\videomass\\icons'
-                self.prg_icon = self.dataloc['icodir'] + "\\videomass.png"
+            sitepkg = os.path.dirname(
+                os.path.dirname(os.path.dirname(__file__)))
+            srcdata = os.path.join(sitepkg, 'videomass', 'data')
+            self.dataloc["app"] = None
+            msg(f"Info: Package: «videomass»\nInfo: Location: «{sitepkg}»")
 
-            elif binarypath == '/usr/local/bin/videomass':
-                msg(f'executable={binarypath}')
-                # pip as super user, usually Linux, MacOs, Unix
-                share = '/usr/local/share/pixmaps'
-                self.prg_icon = share + '/videomass.png'
+        self.dataloc["localepath"] = os.path.join(srcdata, 'locale')
+        self.dataloc["srcdata"] = srcdata
+        self.dataloc["icodir"] = os.path.join(srcdata, 'icons')
+        self.dataloc["FFMPEG_DIR"] = os.path.join(srcdata, 'FFMPEG')
 
-            elif binarypath == '/usr/bin/videomass':
-                msg(f'executable={binarypath}')
-                # installed via apt, rpm, etc, usually Linux
-                share = '/usr/share/pixmaps'
-                self.prg_icon = share + "/videomass.png"
-
-            else:
-                msg(f'executable={binarypath}')
-                # pip as user, usually Linux, MacOs, Unix
-                if binarypath is None:
-                    # need if user $PATH is not set yet
-                    userbase = site.getuserbase()
-                else:
-                    userbase = os.path.dirname(os.path.dirname(binarypath))
-                pixmaps = '/share/pixmaps/videomass.png'
-                self.prg_icon = os.path.join(userbase + pixmaps)
+        self.prg_icon = os.path.join(self.dataloc['icodir'], "videomass.png")
     # ---------------------------------------------------------------------
 
-    def get_fileconf(self):
+    def get_configuration(self):
         """
-        Get settings.json configuration data and returns a dict object
-        with current data-set for bootstrap.
+        Get configuration data of the application.
+        Returns a dict object with current data-set for bootstrap.
 
         Note: If returns a dict key == ERROR it will raise a windowed
         fatal error in the gui_app bootstrap.
         """
+        # checks configuration directory
+        ckdconf = restore_dirconf(self.dataloc['confdir'],
+                                  self.dataloc['srcdata'],
+                                  self.makeportable,
+                                  )
+        if ckdconf.get('ERROR'):
+            return ckdconf
+
         # handle configuration file
-        userconf = get_options(self.dataloc['confdir'],
-                               self.dataloc['conffile'],
-                               self.dataloc['srcpath'],
-                               self.makeportable,
-                               )
+        userconf = get_options(self.dataloc['conffile'], self.makeportable)
         if userconf.get('ERROR'):
             return userconf
         userconf = userconf['R']
 
-        # restore presets folder
-        presets_rest = restore_presets_dir(self.dataloc['confdir'],
-                                           self.dataloc['srcpath'],
-                                           )
-        if presets_rest.get('ERROR'):
-            return presets_rest
-
         # create the required directories if not existing
         requiredirs = (os.path.join(self.dataloc['cachedir'], 'tmp'),
                        self.dataloc['logdir'],
-                       userconf['outputdir'],
-                       userconf['ydlp-outputdir'],
                        self.dataloc['trashdir_default']
                        )
         if not userconf['trashdir_loc'].strip():
@@ -442,22 +352,20 @@ class DataSource():
             try:
                 return os.path.relpath(path) if relative else path
             except (ValueError, TypeError):
-                # return {'ERROR': f'{error}'}  # use `as error` here
+                # return {'ERROR': f'{error}'}  # used `as error` here
                 return path
 
         return ({'ostype': platform.system(),
-                 'srcpath': _relativize(self.dataloc['srcpath']),
+                 'app': self.dataloc['app'],
+                 'srcdata': _relativize(self.dataloc['srcdata']),
                  'localepath': _relativize(self.dataloc['localepath']),
                  'fileconfpath': _relativize(self.dataloc['conffile']),
-                 'workdir': _relativize(self.dataloc['workdir']),
                  'confdir': _relativize(self.dataloc['confdir']),
                  'logdir': _relativize(self.dataloc['logdir']),
                  'cachedir': _relativize(self.dataloc['cachedir']),
                  'trashdir_default':
                      _relativize(self.dataloc['trashdir_default']),
-                 'FFMPEG_videomass_pkg':
-                     _relativize(self.dataloc['ffmpeg_pkg']),
-                 'app': self.apptype,
+                 'FFMPEG_DIR': _relativize(self.dataloc['FFMPEG_DIR']),
                  'relpath': self.relativepaths,
                  'getpath': _relativize,
                  'ffmpeg_cmd': _relativize(userconf['ffmpeg_cmd']),
@@ -465,17 +373,17 @@ class DataSource():
                  'ffplay_cmd': _relativize(userconf['ffplay_cmd']),
                  'ffmpeg-default-args': '-y -stats -hide_banner',
                  'ffplay-default-args': '-hide_banner',
+                 'auto-restart-app': False,
+                 'make_portable': self.makeportable,
                  **userconf
                  })
     # --------------------------------------------------------------------
 
     def icons_set(self, icontheme):
         """
-        Determines icons set assignment defined on the configuration
-        file (see `Set icon themes map:`, on paragraph `6- GUI setup`
-        in the settings.json file).
+        Determines icons set assignment defined on the
+        configuration file.
         Returns a icontheme dict object.
-
         """
         keys = ('videomass', 'A/V-Conv', 'startconv', 'fileproperties',
                 'playback', 'concatenate', 'preview', 'clear',
@@ -485,9 +393,11 @@ class DataSource():
                 'profile_edit', 'previous', 'next', 'stabilizer',
                 'preview_audio', 'profile_copy', 'slideshow',
                 'videotopictures', 'atrack', 'timerset', 'coloreq',
-                'stop', 'home', 'youtube', 'playlist',
-                'download', 'statistics', 'play', 'subtitles',
-                'proc-queue', 'add-queue', 'options',
+                'stop', 'home', 'play', 'proc-queue',
+                'add-queue', 'delpreset', 'exportall', 'exportselected',
+                'importfolder', 'importpreset', 'newpreset', 'reload',
+                'restoreall', 'restoreselected', 'cmdshow', 'start_time',
+                'end_time', 'waveform'
                 )  # must match with items on `iconset` tuple, see following
 
         icodir = self.dataloc['icodir']
@@ -530,7 +440,7 @@ class DataSource():
                    f"{choose.get('x16')}/deinterlace.{ext}",
                    f"{choose.get('x16')}/denoise.{ext}",
                    f"{choose.get('x16')}/volanalyze.{ext}",
-                   f"{choose.get('x16')}/configure.{ext}",
+                   f"{choose.get('x16')}/settings.{ext}",
                    f"{choose.get('x16')}/player-volume.{ext}",
                    f"{choose.get('x48')}/icon_prst_mng.{ext}",
                    f"{choose.get('x16')}/newprf.{ext}",
@@ -548,15 +458,22 @@ class DataSource():
                    f"{choose.get('x16')}/coloreq.{ext}",
                    f"{choose.get('x22')}/stop.{ext}",
                    f"{choose.get('x22')}/home.{ext}",
-                   f"{choose.get('x48')}/youtube.{ext}",
-                   f"{choose.get('x16')}/playlist.{ext}",
-                   f"{choose.get('x22')}/download.{ext}",
-                   f"{choose.get('x22')}/statistics.{ext}",
                    f"{choose.get('x22')}/play.{ext}",
-                   f"{choose.get('x16')}/subtitles.{ext}",
                    f"{choose.get('x22')}/proc-queue.{ext}",
                    f"{choose.get('x22')}/add-queue.{ext}",
-                   f"{choose.get('x22')}/options.{ext}",
+                   f"{choose.get('x16')}/delpreset.{ext}",
+                   f"{choose.get('x16')}/exportall.{ext}",
+                   f"{choose.get('x16')}/exportselected.{ext}",
+                   f"{choose.get('x16')}/importfolder.{ext}",
+                   f"{choose.get('x16')}/importpreset.{ext}",
+                   f"{choose.get('x16')}/newpreset.{ext}",
+                   f"{choose.get('x16')}/reload.{ext}",
+                   f"{choose.get('x16')}/restoreall.{ext}",
+                   f"{choose.get('x16')}/restoreselected.{ext}",
+                   f"{choose.get('x16')}/cmdshow.{ext}",
+                   f"{choose.get('x16')}/start_time.{ext}",
+                   f"{choose.get('x16')}/end_time.{ext}",
+                   f"{choose.get('x16')}/waveform.{ext}",
                    )
         values = (os.path.join(norm) for norm in iconset)  # normalize pathns
 
